@@ -162,7 +162,7 @@ class _ExprTypeChecker:
 
     def types_from_Compare(self, node):
         # comparison: `x < y`
-        if isinstance(node.op, vy_ast.In):
+        if isinstance(node.op, (vy_ast.In, vy_ast.NotIn)):
             # x in y
             left = self.get_possible_types_from_node(node.left)
             right = self.get_possible_types_from_node(node.right)
@@ -441,3 +441,20 @@ def get_index_value(node: vy_ast.Index) -> int:
         raise ArrayIndexException("Subscript must be greater than 0", node)
 
     return node.value.value
+
+
+def validate_unique_method_ids(functions: List) -> None:
+    """
+    Check for collisions between the 4byte function selectors
+    of each function within a contract.
+
+    Arguments
+    ---------
+    functions : List[ContractFunction]
+        A list of ContractFunction objects.
+    """
+    method_ids = [x for i in functions for x in i.method_ids.values()]
+    collision = next((i for i in method_ids if method_ids.count(i) > 1), None)
+    if collision:
+        collision_str = ", ".join(i.name for i in functions if collision in i.method_ids)
+        raise StructureException(f"Methods have conflicting IDs: {collision_str}")
