@@ -18,6 +18,7 @@ from vyper.parser.parser_utils import (
     add_variable_offset,
     get_number_as_fraction,
     getpos,
+    make_byte_array_copier,
     make_setter,
     unwrap_location,
 )
@@ -39,7 +40,6 @@ from vyper.utils import (
     MemoryPositions,
     SizeLimits,
     bytes_to_int,
-    check_valid_varname,
     checksum_encode,
     string_to_bytes,
 )
@@ -255,7 +255,7 @@ class Expr:
         return self._make_bytelike(typ, bytez, bytez_length)
 
     def _make_bytelike(self, btype, bytez, bytez_length):
-        placeholder = self.context.new_placeholder(btype)
+        placeholder = self.context.new_internal_variable(btype)
         seq = []
         seq.append(["mstore", placeholder, bytez_length])
         for i in range(0, len(bytez), 32):
@@ -412,6 +412,29 @@ class Expr:
         sub = Expr.parse_variable_location(self.expr.value, self.context)
         if isinstance(sub.typ, (MappingType, ListType)):
             index = Expr.parse_value_expr(self.expr.slice.value, self.context)
+            if isinstance(index.typ, ByteArrayLike) and index.args[0].location == "storage":
+                # Special case - if the key value is a bytes-array type located in
+                # storage, we have to copy it to memory prior to calculating the hash
+                placeholder = self.context.new_internal_variable(index.typ)
+                placeholder_node = LLLnode.from_list(placeholder, typ=index.typ, location="memory")
+                copier = make_byte_array_copier(
+                    placeholder_node,
+                    LLLnode.from_list(index.args[0], typ=index.typ, location="storage"),
+                )
+                return LLLnode.from_list(
+                    [
+                        "seq",
+                        copier,
+                        [
+                            "sha3_64",
+                            sub,
+                            ["sha3", ["add", placeholder, 32], ["mload", placeholder]],
+                        ],
+                    ],
+                    typ=sub.typ.valuetype,
+                    pos=getpos(self.expr),
+                    location="storage",
+                )
         elif isinstance(sub.typ, TupleType):
             index = self.expr.slice.value.n
             if not 0 <= index < len(sub.typ.members):
@@ -613,14 +636,16 @@ class Expr:
         left = Expr(self.expr.left, self.context).lll_node
         right = Expr(self.expr.right, self.context).lll_node
 
-        result_placeholder = self.context.new_placeholder(BaseType("bool"))
+        result_placeholder = self.context.new_internal_variable(BaseType("bool"))
         setter = []
 
         # Load nth item from list in memory.
         if right.value == "multi":
             # Copy literal to memory to be compared.
             tmp_list = LLLnode.from_list(
-                obj=self.context.new_placeholder(ListType(right.typ.subtype, right.typ.count)),
+                obj=self.context.new_internal_variable(
+                    ListType(right.typ.subtype, right.typ.count)
+                ),
                 typ=ListType(right.typ.subtype, right.typ.count),
                 location="memory",
             )
@@ -673,12 +698,11 @@ class Expr:
         else:
             compare_sequence = ["seq"] + for_loop_sequence
 
-        # Compare the result of the repeat loop to 1, to know if a match was found.
-        lll_node = LLLnode.from_list(
-            ["eq", 1, compare_sequence], typ="bool", annotation="in comporator"
-        )
+        if isinstance(self.expr.op, vy_ast.NotIn):
+            # for `not in`, invert the result
+            compare_sequence = ["iszero", compare_sequence]
 
-        return lll_node
+        return LLLnode.from_list(compare_sequence, typ="bool", annotation="in comparator")
 
     @staticmethod
     def _signed_to_unsigned_comparision_op(op):
@@ -704,7 +728,9 @@ class Expr:
             # TODO: Can this if branch be removed ^
             pass
 
-        elif isinstance(self.expr.op, vy_ast.In) and isinstance(right.typ, ListType):
+        elif isinstance(self.expr.op, (vy_ast.In, vy_ast.NotIn)) and isinstance(
+            right.typ, ListType
+        ):
             if left.typ != right.typ.subtype:
                 return
             return self.build_in_comparator()
@@ -912,20 +938,15 @@ class Expr:
             return lll_node.typ
 
         lll_node = []
-        previous_type = None
         out_type = None
 
         for elt in self.expr.elements:
             current_lll_node = Expr(elt, self.context).lll_node
-            if not out_type:
+            if not out_type or not current_lll_node.typ.is_literal:
+                # prefer to use a non-literal type here, because literals can be ambiguous
+                # this should be removed altogether as we refactor types out of parser
                 out_type = current_lll_node.typ
-
-            current_type = get_out_type(current_lll_node)
-            if len(lll_node) > 0 and previous_type != current_type:
-                raise TypeMismatch("Lists may only contain one type", self.expr)
-            else:
-                lll_node.append(current_lll_node)
-                previous_type = current_type
+            lll_node.append(current_lll_node)
 
         return LLLnode.from_list(
             ["multi"] + lll_node, typ=ListType(out_type, len(lll_node)), pos=getpos(self.expr),
@@ -938,9 +959,6 @@ class Expr:
         for key, value in zip(expr.keys, expr.values):
             if not isinstance(key, vy_ast.Name):
                 return
-            check_valid_varname(
-                key.id, context.structs, "Invalid member variable for struct",
-            )
             if key.id in member_subs:
                 return
             sub = Expr(value, context).lll_node
@@ -963,7 +981,7 @@ class Expr:
                 # assign it's result to memory - otherwise there is potential for memory corruption
                 lll_node = Expr(node, self.context).lll_node
                 target = LLLnode.from_list(
-                    self.context.new_placeholder(lll_node.typ),
+                    self.context.new_internal_variable(lll_node.typ),
                     typ=lll_node.typ,
                     location="memory",
                     pos=getpos(self.expr),
