@@ -1,7 +1,6 @@
-import ast
 from typing import Any, List
 
-from vyper.exceptions import FunctionDeclarationException
+from vyper import ast as vy_ast
 from vyper.parser.context import Context
 from vyper.parser.expr import Expr
 from vyper.parser.function_definitions.utils import (
@@ -35,14 +34,8 @@ def get_internal_arg_copier(total_size: int, memory_dest: int) -> List[Any]:
     return copier
 
 
-def validate_internal_function(code: ast.FunctionDef, sig: FunctionSignature) -> None:
-    """ Validate internal function defintion """
-    if sig.is_default_func():
-        raise FunctionDeclarationException("Default function may only be external.", code)
-
-
 def parse_internal_function(
-    code: ast.FunctionDef, sig: FunctionSignature, context: Context
+    code: vy_ast.FunctionDef, sig: FunctionSignature, context: Context
 ) -> LLLnode:
     """
     Parse a internal function (FuncDef), and produce full function body.
@@ -52,20 +45,20 @@ def parse_internal_function(
     :return: full sig compare & function body
     """
 
-    validate_internal_function(code, sig)
+    func_type = code._metadata["type"]
 
     # Get nonreentrant lock
-    nonreentrant_pre, nonreentrant_post = get_nonreentrant_lock(sig, context.global_ctx)
+    nonreentrant_pre, nonreentrant_post = get_nonreentrant_lock(func_type, context.global_ctx)
 
     # Create callback_ptr, this stores a destination in the bytecode for a internal
     # function to jump to after a function has executed.
     clampers: List[LLLnode] = []
 
     # Allocate variable space.
-    context.memory_allocator.increase_memory(sig.max_copy_size)
+    context.memory_allocator.expand_memory(sig.max_copy_size)
 
     _post_callback_ptr = f"{sig.name}_{sig.method_id}_post_callback_ptr"
-    context.callback_ptr = context.new_placeholder(typ=BaseType("uint256"))
+    context.callback_ptr = context.new_internal_variable(typ=BaseType("uint256"))
     clampers.append(
         LLLnode.from_list(
             ["mstore", context.callback_ptr, "pass"], annotation="pop callback pointer",
@@ -95,7 +88,7 @@ def parse_internal_function(
     # Fill variable positions
     for arg in sig.args:
         if isinstance(arg.typ, ByteArrayLike):
-            mem_pos = context.memory_allocator.increase_memory(32 * get_size_of_type(arg.typ))
+            mem_pos = context.memory_allocator.expand_memory(32 * get_size_of_type(arg.typ))
             context.vars[arg.name] = VariableRecord(arg.name, mem_pos, arg.typ, False)
         else:
             context.vars[arg.name] = VariableRecord(
@@ -105,7 +98,7 @@ def parse_internal_function(
     # internal function copiers. No clamping for internal functions.
     dyn_variable_names = [a.name for a in sig.base_args if isinstance(a.typ, ByteArrayLike)]
     if dyn_variable_names:
-        i_placeholder = context.new_placeholder(typ=BaseType("uint256"))
+        i_placeholder = context.new_internal_variable(typ=BaseType("uint256"))
         unpackers: List[Any] = []
         for idx, var_name in enumerate(dyn_variable_names):
             var = context.vars[var_name]
@@ -182,7 +175,7 @@ def parse_internal_function(
 
                 # Unpack byte array if necessary.
                 if dynamics:
-                    i_placeholder = context.new_placeholder(typ=BaseType("uint256"))
+                    i_placeholder = context.new_internal_variable(typ=BaseType("uint256"))
                     for idx, var_pos in enumerate(dynamics):
                         ident = f"unpack_default_sig_dyn_{default_sig.method_id}_arg{idx}"
                         default_copiers.append(
@@ -216,23 +209,16 @@ def parse_internal_function(
         _clampers = [["label", _post_callback_ptr]]
 
         # Function with default parameters.
-        o = LLLnode.from_list(
+        return LLLnode.from_list(
             [
                 "seq",
                 sig_chain,
-                [
-                    "if",
-                    0,  # can only be jumped into
-                    [
-                        "seq",
-                        ["seq"]
-                        + nonreentrant_pre
-                        + _clampers
-                        + [parse_body(c, context) for c in code.body]
-                        + nonreentrant_post
-                        + stop_func,
-                    ],
-                ],
+                ["seq"]
+                + nonreentrant_pre
+                + _clampers
+                + [parse_body(c, context) for c in code.body]
+                + nonreentrant_post
+                + stop_func,
             ],
             typ=None,
             pos=getpos(code),
@@ -241,21 +227,14 @@ def parse_internal_function(
     else:
         # Function without default parameters.
         sig_compare, internal_label = get_sig_statements(sig, getpos(code))
-        o = LLLnode.from_list(
-            [
-                "if",
-                sig_compare,
-                ["seq"]
-                + [internal_label]
-                + nonreentrant_pre
-                + clampers
-                + [parse_body(c, context) for c in code.body]
-                + nonreentrant_post
-                + stop_func,
-            ],
+        return LLLnode.from_list(
+            ["seq"]
+            + [internal_label]
+            + nonreentrant_pre
+            + clampers
+            + [parse_body(c, context) for c in code.body]
+            + nonreentrant_post
+            + stop_func,
             typ=None,
             pos=getpos(code),
         )
-        return o
-
-    return o
