@@ -8,7 +8,6 @@ from vyper.parser.events import pack_logging_data, pack_logging_topics
 from vyper.parser.expr import Expr
 from vyper.parser.parser_utils import (
     LLLnode,
-    base_type_conversion,
     getpos,
     make_byte_array_copier,
     make_setter,
@@ -38,7 +37,9 @@ class Stmt:
         if fn is None:
             raise TypeCheckFailure(f"Invalid statement node: {type(node).__name__}")
 
-        self.lll_node = fn()
+        with context.internal_memory_scope():
+            self.lll_node = fn()
+
         if self.lll_node is None:
             raise TypeCheckFailure("Statement node did not produce LLL")
 
@@ -55,98 +56,91 @@ class Stmt:
             raise StructureException(f"Unsupported statement type: {type(self.stmt)}", self.stmt)
 
     def parse_AnnAssign(self):
-        with self.context.assignment_scope():
-            typ = parse_type(
-                self.stmt.annotation, location="memory", custom_structs=self.context.structs,
-            )
-            varname = self.stmt.target.id
-            pos = self.context.new_variable(varname, typ, pos=self.stmt)
-            if self.stmt.value is None:
-                return
+        typ = parse_type(
+            self.stmt.annotation, location="memory", custom_structs=self.context.structs,
+        )
+        varname = self.stmt.target.id
+        pos = self.context.new_variable(varname, typ, pos=self.stmt)
+        if self.stmt.value is None:
+            return
 
-            sub = Expr(self.stmt.value, self.context).lll_node
+        sub = Expr(self.stmt.value, self.context).lll_node
 
-            is_literal_bytes32_assign = (
-                isinstance(sub.typ, ByteArrayType)
-                and sub.typ.maxlen == 32
-                and isinstance(typ, BaseType)
-                and typ.typ == "bytes32"
-                and sub.typ.is_literal
+        is_literal_bytes32_assign = (
+            isinstance(sub.typ, ByteArrayType)
+            and sub.typ.maxlen == 32
+            and isinstance(typ, BaseType)
+            and typ.typ == "bytes32"
+            and sub.typ.is_literal
+        )
+
+        # If bytes[32] to bytes32 assignment rewrite sub as bytes32.
+        if is_literal_bytes32_assign:
+            sub = LLLnode(
+                bytes_to_int(self.stmt.value.s), typ=BaseType("bytes32"), pos=getpos(self.stmt),
             )
 
-            # If bytes[32] to bytes32 assignment rewrite sub as bytes32.
-            if is_literal_bytes32_assign:
-                sub = LLLnode(
-                    bytes_to_int(self.stmt.value.s), typ=BaseType("bytes32"), pos=getpos(self.stmt),
-                )
+        variable_loc = LLLnode.from_list(pos, typ=typ, location="memory", pos=getpos(self.stmt),)
+        lll_node = make_setter(variable_loc, sub, "memory", pos=getpos(self.stmt))
 
-            variable_loc = LLLnode.from_list(
-                pos, typ=typ, location="memory", pos=getpos(self.stmt),
-            )
-            lll_node = make_setter(variable_loc, sub, "memory", pos=getpos(self.stmt))
-
-            return lll_node
+        return lll_node
 
     def parse_Assign(self):
         # Assignment (e.g. x[4] = y)
-        with self.context.assignment_scope():
-            sub = Expr(self.stmt.value, self.context).lll_node
-            target = self._get_target(self.stmt.target)
-            lll_node = make_setter(target, sub, target.location, pos=getpos(self.stmt))
-            lll_node.pos = getpos(self.stmt)
+        sub = Expr(self.stmt.value, self.context).lll_node
+        target = self._get_target(self.stmt.target)
+        lll_node = make_setter(target, sub, target.location, pos=getpos(self.stmt))
+        lll_node.pos = getpos(self.stmt)
         return lll_node
 
     def parse_If(self):
         if self.stmt.orelse:
-            block_scope_id = id(self.stmt.orelse)
-            with self.context.make_blockscope(block_scope_id):
+            with self.context.block_scope():
                 add_on = [parse_body(self.stmt.orelse, self.context)]
         else:
             add_on = []
 
-        block_scope_id = id(self.stmt)
-        with self.context.make_blockscope(block_scope_id):
+        with self.context.block_scope():
             test_expr = Expr.parse_value_expr(self.stmt.test, self.context)
             body = ["if", test_expr, parse_body(self.stmt.body, self.context)] + add_on
             lll_node = LLLnode.from_list(body, typ=None, pos=getpos(self.stmt))
         return lll_node
 
     def parse_Log(self):
-        event = self.context.sigs["self"][self.stmt.value.func.id]
+        event = self.stmt._metadata["type"]
+        args = self.stmt.value.args
+
         expected_topics, topics = [], []
         expected_data, data = [], []
-        with self.context.internal_memory_scope(id(self.stmt)):
-            for pos, is_indexed in enumerate(event.indexed_list):
-                if is_indexed:
-                    expected_topics.append(event.args[pos])
-                    topics.append(self.stmt.value.args[pos])
-                else:
-                    expected_data.append(event.args[pos])
-                    data.append(self.stmt.value.args[pos])
-            topics = pack_logging_topics(
-                event.event_id, topics, expected_topics, self.context, pos=getpos(self.stmt),
-            )
-            inargs, inargsize, inargsize_node, inarg_start = pack_logging_data(
-                expected_data, data, self.context, pos=getpos(self.stmt),
-            )
-
-            if inargsize_node is None:
-                sz = inargsize
+        for node, arg_type, is_indexed in zip(args, event.arguments.values(), event.indexed):
+            if is_indexed:
+                expected_topics.append(arg_type)
+                topics.append(node)
             else:
-                sz = ["mload", inargsize_node]
+                expected_data.append(arg_type)
+                data.append(node)
+        topics = pack_logging_topics(event.event_id, topics, expected_topics, self.context)
+        inargs, inargsize, inargsize_node, inarg_start = pack_logging_data(
+            data, expected_data, self.context, getpos(self.stmt),
+        )
 
-            return LLLnode.from_list(
-                [
-                    "seq",
-                    inargs,
-                    LLLnode.from_list(
-                        ["log" + str(len(topics)), inarg_start, sz] + topics,
-                        add_gas_estimate=inargsize * 10,
-                    ),
-                ],
-                typ=None,
-                pos=getpos(self.stmt),
-            )
+        if inargsize_node is None:
+            sz = inargsize
+        else:
+            sz = ["mload", inargsize_node]
+
+        return LLLnode.from_list(
+            [
+                "seq",
+                inargs,
+                LLLnode.from_list(
+                    ["log" + str(len(topics)), inarg_start, sz] + topics,
+                    add_gas_estimate=inargsize * 10,
+                ),
+            ],
+            typ=None,
+            pos=getpos(self.stmt),
+        )
 
     def parse_Call(self):
         is_self_function = (
@@ -167,29 +161,36 @@ class Stmt:
         if isinstance(msg, vy_ast.Name) and msg.id == "UNREACHABLE":
             return LLLnode.from_list(["assert_unreachable", test_expr], typ=None, pos=getpos(msg))
 
-        with self.context.internal_memory_scope(id(self.stmt)):
-            reason_str = msg.s.strip()
-            sig_placeholder = self.context.new_placeholder(BaseType(32))
-            arg_placeholder = self.context.new_placeholder(BaseType(32))
-            reason_str_type = ByteArrayType(len(reason_str))
-            placeholder_bytes = Expr(msg, self.context).lll_node
-            method_id = fourbytes_to_int(keccak256(b"Error(string)")[:4])
-            assert_reason = [
-                "seq",
-                ["mstore", sig_placeholder, method_id],
-                ["mstore", arg_placeholder, 32],
-                placeholder_bytes,
-                [
-                    "assert_reason",
-                    test_expr,
-                    int(sig_placeholder + 28),
-                    int(4 + get_size_of_type(reason_str_type) * 32),
-                ],
-            ]
-            return LLLnode.from_list(assert_reason, typ=None, pos=getpos(self.stmt))
+        reason_str_type = ByteArrayType(len(msg.value.strip()))
+
+        sig_placeholder = self.context.new_internal_variable(BaseType(32))
+        arg_placeholder = self.context.new_internal_variable(BaseType(32))
+        placeholder_bytes = Expr(msg, self.context).lll_node
+
+        method_id = fourbytes_to_int(keccak256(b"Error(string)")[:4])
+
+        revert_seq = [
+            "seq",
+            ["mstore", sig_placeholder, method_id],
+            ["mstore", arg_placeholder, 32],
+            placeholder_bytes,
+            ["revert", sig_placeholder + 28, int(4 + get_size_of_type(reason_str_type) * 32)],
+        ]
+        if test_expr:
+            lll_node = ["if", ["iszero", test_expr], revert_seq]
+        else:
+            lll_node = revert_seq
+
+        return LLLnode.from_list(lll_node, typ=None, pos=getpos(self.stmt))
 
     def parse_Assert(self):
         test_expr = Expr.parse_value_expr(self.stmt.test, self.context)
+        if test_expr.typ.is_literal:
+            if test_expr.value == 1:
+                # skip literal assertions that always pass
+                return LLLnode.from_list(["pass"], typ=None, pos=getpos(self.stmt))
+            else:
+                test_expr = test_expr.value
 
         if self.stmt.msg:
             return self._assert_reason(test_expr, self.stmt.msg)
@@ -200,7 +201,7 @@ class Stmt:
         if self.stmt.exc:
             return self._assert_reason(0, self.stmt.exc)
         else:
-            return LLLnode.from_list(["assert", 0], typ=None, pos=getpos(self.stmt))
+            return LLLnode.from_list(["revert", 0, 0], typ=None, pos=getpos(self.stmt))
 
     def _check_valid_range_constant(self, arg_ast_node, raise_exception=True):
         with self.context.range_scope():
@@ -224,84 +225,60 @@ class Stmt:
         return arg_expr.value
 
     def parse_For(self):
-        # Type 0 for, e.g. for i in list(): ...
-        if self._is_list_iter():
-            return self._parse_for_list()
-
-        if self.stmt.get("iter.func.id") != "range":
-            return
-        if not 0 < len(self.stmt.iter.args) < 3:
-            return
+        with self.context.block_scope():
+            if self.stmt.get("iter.func.id") == "range":
+                return self._parse_For_range()
+            else:
+                return self._parse_For_list()
 
+    def _parse_For_range(self):
         # attempt to use the type specified by type checking, fall back to `int128`
         # this is a stopgap solution to allow uint256 - it will be properly solved
         # once we refactor `vyper.parser`
-        iter_typ = self.stmt.target.get("_type") or "int128"
-        block_scope_id = id(self.stmt)
-        with self.context.make_blockscope(block_scope_id):
-            # Get arg0
-            arg0 = self.stmt.iter.args[0]
-            num_of_args = len(self.stmt.iter.args)
+        iter_typ = "int128"
+        if "type" in self.stmt.target._metadata:
+            iter_typ = self.stmt.target._metadata["type"]._id
 
-            # Type 1 for, e.g. for i in range(10): ...
-            if num_of_args == 1:
-                arg0_val = self._get_range_const_value(arg0)
-                start = LLLnode.from_list(0, typ=iter_typ, pos=getpos(self.stmt))
-                rounds = arg0_val
+        # Get arg0
+        arg0 = self.stmt.iter.args[0]
+        num_of_args = len(self.stmt.iter.args)
 
-            # Type 2 for, e.g. for i in range(100, 110): ...
-            elif self._check_valid_range_constant(self.stmt.iter.args[1], raise_exception=False)[0]:
-                arg0_val = self._get_range_const_value(arg0)
-                arg1_val = self._get_range_const_value(self.stmt.iter.args[1])
-                start = LLLnode.from_list(arg0_val, typ=iter_typ, pos=getpos(self.stmt))
-                rounds = LLLnode.from_list(arg1_val - arg0_val, typ=iter_typ, pos=getpos(self.stmt))
+        # Type 1 for, e.g. for i in range(10): ...
+        if num_of_args == 1:
+            arg0_val = self._get_range_const_value(arg0)
+            start = LLLnode.from_list(0, typ=iter_typ, pos=getpos(self.stmt))
+            rounds = arg0_val
 
-            # Type 3 for, e.g. for i in range(x, x + 10): ...
-            else:
-                arg1 = self.stmt.iter.args[1]
-                rounds = self._get_range_const_value(arg1.right)
-                start = Expr.parse_value_expr(arg0, self.context)
+        # Type 2 for, e.g. for i in range(100, 110): ...
+        elif self._check_valid_range_constant(self.stmt.iter.args[1], raise_exception=False)[0]:
+            arg0_val = self._get_range_const_value(arg0)
+            arg1_val = self._get_range_const_value(self.stmt.iter.args[1])
+            start = LLLnode.from_list(arg0_val, typ=iter_typ, pos=getpos(self.stmt))
+            rounds = LLLnode.from_list(arg1_val - arg0_val, typ=iter_typ, pos=getpos(self.stmt))
 
-            r = rounds if isinstance(rounds, int) else rounds.value
-            if r < 1:
-                return
+        # Type 3 for, e.g. for i in range(x, x + 10): ...
+        else:
+            arg1 = self.stmt.iter.args[1]
+            rounds = self._get_range_const_value(arg1.right)
+            start = Expr.parse_value_expr(arg0, self.context)
 
-            varname = self.stmt.target.id
-            pos = self.context.new_variable(varname, BaseType(iter_typ), pos=getpos(self.stmt))
-            self.context.forvars[varname] = True
-            lll_node = LLLnode.from_list(
-                ["repeat", pos, start, rounds, parse_body(self.stmt.body, self.context)],
-                typ=None,
-                pos=getpos(self.stmt),
-            )
-            del self.context.vars[varname]
-            del self.context.forvars[varname]
+        r = rounds if isinstance(rounds, int) else rounds.value
+        if r < 1:
+            return
+
+        varname = self.stmt.target.id
+        pos = self.context.new_variable(varname, BaseType(iter_typ), pos=getpos(self.stmt))
+        self.context.forvars[varname] = True
+        lll_node = LLLnode.from_list(
+            ["repeat", pos, start, rounds, parse_body(self.stmt.body, self.context)],
+            typ=None,
+            pos=getpos(self.stmt),
+        )
+        del self.context.forvars[varname]
 
         return lll_node
 
-    def _is_list_iter(self):
-        """
-        Test if the current statement is a type of list, used in for loops.
-        """
-
-        # Check for literal or memory list.
-        iter_var_type = (
-            self.context.vars.get(self.stmt.iter.id).typ
-            if isinstance(self.stmt.iter, vy_ast.Name)
-            else None
-        )
-        if isinstance(self.stmt.iter, vy_ast.List) or isinstance(iter_var_type, ListType):
-            return True
-
-        # Check for storage list.
-        if isinstance(self.stmt.iter, vy_ast.Attribute):
-            iter_var_type = self.context.globals.get(self.stmt.iter.attr)
-            if iter_var_type and isinstance(iter_var_type.typ, ListType):
-                return True
-
-        return False
-
-    def _parse_for_list(self):
+    def _parse_For_list(self):
         with self.context.range_scope():
             iter_list_node = Expr(self.stmt.iter, self.context).lll_node
         if not isinstance(iter_list_node.typ.subtype, BaseType):  # Sanity check on list subtype.
@@ -314,42 +291,37 @@ class Stmt:
         subtype = iter_list_node.typ.subtype.typ
         varname = self.stmt.target.id
         value_pos = self.context.new_variable(varname, BaseType(subtype),)
-        i_pos_raw_name = "_index_for_" + varname
-        i_pos = self.context.new_internal_variable(i_pos_raw_name, BaseType(subtype),)
+        i_pos = self.context.new_internal_variable(BaseType(subtype))
         self.context.forvars[varname] = True
 
         # Is a list that is already allocated to memory.
         if iter_var_type:
-
-            list_name = self.stmt.iter.id
-            # make sure list cannot be altered whilst iterating.
-            with self.context.in_for_loop_scope(list_name):
-                iter_var = self.context.vars.get(self.stmt.iter.id)
-                if iter_var.location == "calldata":
-                    fetcher = "calldataload"
-                elif iter_var.location == "memory":
-                    fetcher = "mload"
-                else:
-                    return
-                body = [
-                    "seq",
-                    [
-                        "mstore",
-                        value_pos,
-                        [fetcher, ["add", iter_var.pos, ["mul", ["mload", i_pos], 32]]],
-                    ],
-                    parse_body(self.stmt.body, self.context),
-                ]
-                lll_node = LLLnode.from_list(
-                    ["repeat", i_pos, 0, iter_var.size, body], typ=None, pos=getpos(self.stmt)
-                )
+            iter_var = self.context.vars.get(self.stmt.iter.id)
+            if iter_var.location == "calldata":
+                fetcher = "calldataload"
+            elif iter_var.location == "memory":
+                fetcher = "mload"
+            else:
+                return
+            body = [
+                "seq",
+                [
+                    "mstore",
+                    value_pos,
+                    [fetcher, ["add", iter_var.pos, ["mul", ["mload", i_pos], 32]]],
+                ],
+                parse_body(self.stmt.body, self.context),
+            ]
+            lll_node = LLLnode.from_list(
+                ["repeat", i_pos, 0, iter_var.size, body], typ=None, pos=getpos(self.stmt)
+            )
 
         # List gets defined in the for statement.
         elif isinstance(self.stmt.iter, vy_ast.List):
             # Allocate list to memory.
             count = iter_list_node.typ.count
             tmp_list = LLLnode.from_list(
-                obj=self.context.new_placeholder(ListType(iter_list_node.typ.subtype, count)),
+                obj=self.context.new_internal_variable(ListType(iter_list_node.typ.subtype, count)),
                 typ=ListType(iter_list_node.typ.subtype, count),
                 location="memory",
             )
@@ -366,28 +338,22 @@ class Stmt:
         # List contained in storage.
         elif isinstance(self.stmt.iter, vy_ast.Attribute):
             count = iter_list_node.typ.count
-            list_name = iter_list_node.annotation
+            body = [
+                "seq",
+                [
+                    "mstore",
+                    value_pos,
+                    ["sload", ["add", ["sha3_32", iter_list_node], ["mload", i_pos]]],
+                ],
+                parse_body(self.stmt.body, self.context),
+            ]
+            lll_node = LLLnode.from_list(
+                ["seq", ["repeat", i_pos, 0, count, body]], typ=None, pos=getpos(self.stmt)
+            )
 
-            # make sure list cannot be altered whilst iterating.
-            with self.context.in_for_loop_scope(list_name):
-                body = [
-                    "seq",
-                    [
-                        "mstore",
-                        value_pos,
-                        ["sload", ["add", ["sha3_32", iter_list_node], ["mload", i_pos]]],
-                    ],
-                    parse_body(self.stmt.body, self.context),
-                ]
-                lll_node = LLLnode.from_list(
-                    ["seq", ["repeat", i_pos, 0, count, body]], typ=None, pos=getpos(self.stmt)
-                )
-
-        del self.context.vars[varname]
         # this kind of open access to the vars dict should be disallowed.
         # we should use member functions to provide an API for these kinds
         # of operations.
-        del self.context.vars[self.context._mangle(i_pos_raw_name)]
         del self.context.forvars[varname]
         return lll_node
 
@@ -410,18 +376,7 @@ class Stmt:
                 self.context,
             )
             return LLLnode.from_list(
-                [
-                    "with",
-                    "_stloc",
-                    target,
-                    [
-                        "sstore",
-                        "_stloc",
-                        base_type_conversion(
-                            lll_node, lll_node.typ, target.typ, pos=getpos(self.stmt)
-                        ),
-                    ],
-                ],
+                ["with", "_stloc", target, ["sstore", "_stloc", unwrap_location(lll_node)]],
                 typ=None,
                 pos=getpos(self.stmt),
             )
@@ -439,18 +394,7 @@ class Stmt:
                 self.context,
             )
             return LLLnode.from_list(
-                [
-                    "with",
-                    "_mloc",
-                    target,
-                    [
-                        "mstore",
-                        "_mloc",
-                        base_type_conversion(
-                            lll_node, lll_node.typ, target.typ, pos=getpos(self.stmt)
-                        ),
-                    ],
-                ],
+                ["with", "_mloc", target, ["mstore", "_mloc", unwrap_location(lll_node)]],
                 typ=None,
                 pos=getpos(self.stmt),
             )
@@ -514,10 +458,10 @@ class Stmt:
                 return
 
             # loop memory has to be allocated first.
-            loop_memory_position = self.context.new_placeholder(typ=BaseType("uint256"))
+            loop_memory_position = self.context.new_internal_variable(typ=BaseType("uint256"))
             # len & bytez placeholder have to be declared after each other at all times.
-            len_placeholder = self.context.new_placeholder(typ=BaseType("uint256"))
-            bytez_placeholder = self.context.new_placeholder(typ=sub.typ)
+            len_placeholder = self.context.new_internal_variable(BaseType("uint256"))
+            bytez_placeholder = self.context.new_internal_variable(sub.typ)
 
             if sub.location in ("storage", "memory"):
                 return LLLnode.from_list(
@@ -545,7 +489,7 @@ class Stmt:
             return
 
         elif isinstance(sub.typ, ListType):
-            loop_memory_position = self.context.new_placeholder(typ=BaseType("uint256"))
+            loop_memory_position = self.context.new_internal_variable(typ=BaseType("uint256"))
             if sub.typ != self.context.return_type:
                 return
             elif sub.location == "memory" and sub.value != "multi":
@@ -563,7 +507,7 @@ class Stmt:
                 )
             else:
                 new_sub = LLLnode.from_list(
-                    self.context.new_placeholder(self.context.return_type),
+                    self.context.new_internal_variable(self.context.return_type),
                     typ=self.context.return_type,
                     location="memory",
                 )
@@ -607,19 +551,6 @@ class Stmt:
             return gen_tuple_return(self.stmt, self.context, sub)
 
     def _get_target(self, target):
-        # Check if we are doing assignment of an iteration loop.
-        if isinstance(target, vy_ast.Subscript) and self.context.in_for_loop:
-            raise_exception = False
-            if isinstance(target.value, vy_ast.Attribute):
-                if f"{target.value.value.id}.{target.value.attr}" in self.context.in_for_loop:
-                    raise_exception = True
-
-            if target.get("value.id") in self.context.in_for_loop:
-                raise_exception = True
-
-            if raise_exception:
-                raise TypeCheckFailure("Failed for-loop constancy check")
-
         if isinstance(target, vy_ast.Name) and target.id in self.context.forvars:
             raise TypeCheckFailure("Failed for-loop constancy check")
 
