@@ -1,41 +1,32 @@
 from collections import Counter
 
 from vyper import ast as vy_ast
-from vyper import parser
-from vyper.exceptions import (
-    FunctionDeclarationException,
-    InvalidType,
-    StructureException,
-)
+from vyper.exceptions import FunctionDeclarationException, StructureException
 from vyper.parser.lll_node import LLLnode
-from vyper.parser.parser_utils import (
-    check_single_exit,
-    check_unmatched_return,
-    getpos,
-)
+from vyper.parser.parser_utils import check_single_exit, getpos
 from vyper.types import (
     ByteArrayLike,
-    StructType,
-    TupleLike,
     TupleType,
     canonicalize_type,
     get_size_of_type,
     parse_type,
 )
-from vyper.utils import (
-    FUNCTION_WHITELIST,
-    check_valid_varname,
-    fourbytes_to_int,
-    is_varname_valid,
-    iterable_cast,
-    keccak256,
-)
+from vyper.utils import fourbytes_to_int, keccak256
 
 
 # Function argument
 class VariableRecord:
     def __init__(
-        self, name, pos, typ, mutable, *, location="memory", blockscopes=None, defined_at=None
+        self,
+        name,
+        pos,
+        typ,
+        mutable,
+        *,
+        location="memory",
+        blockscopes=None,
+        defined_at=None,
+        is_internal=False,
     ):
         self.name = name
         self.pos = pos
@@ -44,9 +35,13 @@ class VariableRecord:
         self.location = location
         self.blockscopes = [] if blockscopes is None else blockscopes
         self.defined_at = defined_at  # source code location variable record was defined.
+        self.is_internal = is_internal
 
     @property
     def size(self):
+        if hasattr(self.typ, "size_in_bytes"):
+            # temporary requirement to support both new and old type objects
+            return self.typ.size_in_bytes
         return get_size_of_type(self.typ)
 
 
@@ -157,35 +152,12 @@ class FunctionSignature:
         name = code.name
         mem_pos = 0
 
-        valid_name, msg = is_varname_valid(name, custom_structs)
-        if not valid_name and (not name.lower() in FUNCTION_WHITELIST):
-            raise FunctionDeclarationException("Function name invalid. " + msg, code)
-
-        # Validate default values.
-        for default_value in getattr(code.args, "defaults", []):
-            validate_default_values(default_value)
-
         # Determine the arguments, expects something of the form def foo(arg1:
         # int128, arg2: int128 ...
         args = []
         for arg in code.args.args:
             # Each arg needs a type specified.
             typ = arg.annotation
-            if not typ:
-                raise InvalidType("Argument must have type", arg)
-            # Validate arg name.
-            check_valid_varname(
-                arg.arg,
-                custom_structs,
-                arg,
-                "Argument name invalid or reserved. ",
-                FunctionDeclarationException,
-            )
-            # Check for duplicate arg name.
-            if arg.arg in (x.name for x in args):
-                raise FunctionDeclarationException(
-                    "Duplicate function argument name: " + arg.arg, arg,
-                )
             parsed_type = parse_type(typ, None, sigs, custom_structs=custom_structs,)
             args.append(
                 VariableRecord(arg.arg, mem_pos, parsed_type, False, defined_at=getpos(arg),)
@@ -197,9 +169,8 @@ class FunctionSignature:
                 mem_pos += get_size_of_type(parsed_type) * 32
 
         mutability = "nonpayable"  # Assume nonpayable by default
-        internal = False
-        external = False
         nonreentrant_key = ""
+        is_internal = False
 
         # Update function properties from decorators
         # NOTE: Can't import enums here because of circular import
@@ -207,27 +178,11 @@ class FunctionSignature:
             if isinstance(dec, vy_ast.Name) and dec.id in ("payable", "view", "pure"):
                 mutability = dec.id
             elif isinstance(dec, vy_ast.Name) and dec.id == "internal":
-                internal = True
+                is_internal = True
             elif isinstance(dec, vy_ast.Name) and dec.id == "external":
-                external = True
+                is_internal = False
             elif isinstance(dec, vy_ast.Call) and dec.func.id == "nonreentrant":
-                if nonreentrant_key:
-                    raise StructureException(
-                        "Only one @nonreentrant decorator allowed per function", dec
-                    )
-                if (
-                    dec.args
-                    and len(dec.args) == 1
-                    and isinstance(dec.args[0], vy_ast.Str)
-                    and dec.args[0].s
-                ):  # noqa: E501
-                    nonreentrant_key = dec.args[0].s
-                else:
-                    raise StructureException(
-                        "@nonreentrant decorator requires a non-empty string to use as a key.", dec
-                    )
-            else:
-                raise StructureException("Bad decorator", dec)
+                nonreentrant_key = dec.args[0].s
 
         if constant_override:
             # In case this override is abused, match previous behavior
@@ -235,40 +190,16 @@ class FunctionSignature:
                 raise StructureException(f"Function {name} cannot be both constant and payable.")
             mutability = "view"
 
-        if external and internal:
-            raise StructureException(
-                f"Cannot use external and internal decorators on the same function: {name}"
-            )
-        if mutability == "payable" and internal:
-            raise StructureException(f"Function {name} cannot be both internal and payable.")
-        if (not external and not internal) and not interface_def:
-            raise StructureException(
-                "Function visibility must be declared (@external or @internal)", code,
-            )
-        if mutability in ("view", "pure") and nonreentrant_key:
-            raise StructureException(
-                f"@nonreentrant makes no sense on a @{mutability} function.", code
-            )
-
         # Determine the return type and whether or not it's constant. Expects something
         # of the form:
         # def foo(): ...
         # def foo() -> int128: ...
         # If there is no return type, ie. it's of the form def foo(): ...
         # and NOT def foo() -> type: ..., then it's null
-        if not code.returns:
-            output_type = None
-        elif isinstance(
-            code.returns, (vy_ast.Name, vy_ast.Compare, vy_ast.Subscript, vy_ast.Call, vy_ast.Tuple)
-        ):
+        output_type = None
+        if code.returns:
             output_type = parse_type(code.returns, None, sigs, custom_structs=custom_structs,)
-        else:
-            raise InvalidType(
-                f"Output type invalid or unsupported: {parse_type(code.returns, None)}",
-                code.returns,
-            )
-        # Output type must be canonicalizable
-        if output_type is not None:
+            # Output type must be canonicalizable
             assert isinstance(output_type, TupleType) or canonicalize_type(output_type)
         # Get the canonical function signature
         sig = cls.get_full_sig(name, code.args.args, sigs, custom_structs)
@@ -280,7 +211,7 @@ class FunctionSignature:
             args,
             output_type,
             mutability,
-            internal,
+            is_internal,
             nonreentrant_key,
             sig,
             method_id,
@@ -288,69 +219,6 @@ class FunctionSignature:
             is_from_json,
         )
 
-    @iterable_cast(dict)
-    def _generate_base_type(self, arg_type, name=""):
-        yield "type", canonicalize_type(arg_type)
-        yield "name", name
-
-    def _generate_param_abi(self, out_arg, name=""):
-        if isinstance(out_arg, TupleLike):
-            return {
-                "type": "tuple",
-                "name": name,
-                "components": [
-                    self._generate_param_abi(
-                        member_type, name=n if type(out_arg) is StructType else "",
-                    )
-                    for n, member_type in out_arg.tuple_items()
-                ],
-            }
-        return self._generate_base_type(arg_type=out_arg, name=name)
-
-    def _generate_outputs_abi(self):
-        if not self.output_type:
-            return []
-        elif isinstance(self.output_type, StructType):
-            # Flatten structs at the base level of output
-            # NOTE: structs inside of structs still encode as tuples
-            return [
-                self._generate_param_abi(typ, name) for name, typ in self.output_type.tuple_items()
-            ]
-        elif isinstance(self.output_type, TupleType):
-            # Flatten tuples at the base level of output
-            return [self._generate_param_abi(x,) for x in self.output_type.members]
-        else:
-            return [self._generate_param_abi(self.output_type)]
-
-    def _generate_inputs_abi(self):
-        if not self.args:
-            return []
-        else:
-            return [self._generate_param_abi(x.typ, name=x.name) for x in self.args]
-
-    def to_abi_dict(self):
-        func_type = "function"
-        if self.name == "__init__":
-            func_type = "constructor"
-        if self.name == "__default__":
-            func_type = "fallback"
-
-        abi_dict = {
-            "name": self.name,
-            "outputs": self._generate_outputs_abi(),
-            "inputs": self._generate_inputs_abi(),
-            "stateMutability": self.mutability,
-            "type": func_type,
-        }
-
-        if self.name in ("__default__", "__init__"):
-            del abi_dict["name"]
-        if self.name == "__default__":
-            del abi_dict["inputs"]
-            del abi_dict["outputs"]
-
-        return abi_dict
-
     @classmethod
     def lookup_sig(cls, sigs, method_name, expr_args, stmt_or_expr, context):
         """
@@ -404,19 +272,4 @@ class FunctionSignature:
 
     def validate_return_statement_balance(self):
         # Run balanced return statement check.
-        check_unmatched_return(self.func_ast_code)
         check_single_exit(self.func_ast_code)
-
-
-def validate_default_values(node):
-    if isinstance(node, vy_ast.Name) and node.id in parser.expr.BUILTIN_CONSTANTS:
-        return
-    if isinstance(node, vy_ast.Attribute) and node.value.id in parser.expr.ENVIRONMENT_VARIABLES:
-        return
-    if not isinstance(node, (vy_ast.Constant, vy_ast.List)):
-        raise FunctionDeclarationException(
-            "Default value must be a literal, built-in constant, or environment variable.", node
-        )
-    if isinstance(node, vy_ast.List):
-        for n in node.elements:
-            validate_default_values(n)
